/*
 * UDS Example Program
 * Copyright (C) 2000, 2001, 2002, 2003  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/init.hh>
#include <uds/refcounter.hh>
#include <iostream>

using namespace std;


// class that uses reference counting

class RefC : public uds::RefCounter<>
{
public:
	RefC() { cout<<"constructor\n"; }
	~RefC() { cout<<"destructor\n"; }
};

typedef uds::GC_Ptr< RefC > GC_Obj;


GC_Obj
new_object()
{
	GC_Obj foo = new RefC();
	// the reference count of the newly created object is now 1
	GC_Obj bar = foo;
	// the reference count is now 2
	
	return foo;
}

int
main()
{
	uds::Init uds_init;
	
	// This object must not be destroyed manually
	// It will be destroyed when the reference count reaches 0.
	GC_Obj x = new_object();
	// the pointers in new_object() were destroyed; the reference
	// count is now 1
	
	return 0;
}
