/*
 * The UDS Collection
 * Copyright (C) 2001, 2002, 2003  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/btrace.hh>
#include <uds/idep.hh>
#include <uds/io_util.hh>
#include <uds/procstream.hh>
#include <uds/str_util.hh>
#include <iomanip>
#include <sstream>
#include <unistd.h>


namespace uds
{

// misc. implementation

namespace
{

using namespace std;


void
btrace_sym( string& bt, bool lb, ProcStream& pr )
{
	void**			ebp = static_cast< void** >( __builtin_frame_address( 1 ) );
	string			func;
	string			sline;
	stringstream	ss;
	
	ss<<"--- UDS backtrace ("<<getpid()<<") ---\n"<<left;
	bt = ss.str();
	
	while ( true )
	{
		ss.str( "" );

		pr<<*( ebp + 1 )<<endl<<flush;
		ebp = static_cast< void** >( *ebp );
		
		if ( !ebp )
			break;
		
		ReadLine( pr, func );
		ReadLine( pr, sline );

		ss<<setw( 20 )<<sline<<"  ";
		if ( lb )
			ss<<BreakLine( func, 22, 79, 79 - ss.str().size() );
		else
			ss<<func<<endl;

		bt += ss.str();
		
		if ( func == "main" )
			break;
	}
}

void
btrace( string& bt, bool res )
{
	void**			ebp = static_cast< void** >( __builtin_frame_address( 1 ) );
	stringstream	ret;
	
	ret<<"--- UDS backtrace ("<<getpid()<<") ---\n";

	if ( res )
		ret<<"(file names and line numbers not available)\n";
	
	while ( true )
	{
		ret<<*( ebp + 1 )<<endl;
		ebp = static_cast< void** >( *ebp );
		
		if ( !ebp )
			break;
	}
	
	bt = ret.str();
}

}

string
Backtrace( bool res, bool lb )
{
	string ret;
	
	if ( res && strlen( ADDRTOLINE ) )
	{
		ProcStream pr;
		Addr2LinePS( pr, true );
		
		if ( pr.Open() )
		{
			btrace_sym( ret, lb, pr );
			return ret;
		}
	}
	
	btrace( ret, res );
	return ret;
}

}
