/*
 * The UDS collection
 * Copyright (C) 1999, 2000, 2001, 2002, 2003  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <cstring>
#include <sys/wait.h>
#include <uds/childdiag.hh>
#include <sstream>
#include "config.h"


namespace uds
{

// ChildDiag implementation

namespace
{

void
add_strsignal( ostream& out, int sig )
{
#ifdef HAVE_STRSIGNAL
	out<<'('<<strsignal( sig )<<")\n";
#endif
}

}

ChildDiag::ChildDiag( int s ) : status( s )
{
}

ChildDiag::~ChildDiag()
{
}

string
ChildDiag::Diagnose() const
{
	stringstream d;
	
	if ( !WIFEXITED( status ) )
		d<<"child didn't exit normally\n";
	else
		d<<"child exited with return value "<<WEXITSTATUS( status )<<endl;
	
	if ( WIFSIGNALED( status ) )
	{
		d<<"process exited because of uncaught signal "
			<<WTERMSIG( status )<<endl;
		add_strsignal( d, WTERMSIG( status ) );
	}
	
	if ( WIFSTOPPED( status ) )
	{
		d<<"child process is currently stopped by signal "
			<<WSTOPSIG( status )<<endl;
		add_strsignal( d, WTERMSIG( status ) );
	}
	
	return d.str();
}

}
