/*
 * The UDS Collection
 * Copyright (C) 2001, 2002, 2003  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <sys/socket.h>
#include <uds/stdexcpt.hh>
#include <uds/fdbuf.hh>
#include <cerrno>
#include <unistd.h>


namespace uds
{

// fdbuf implementation

fdbuf::fdbuf() :
	fd( -1 )
{
}

fdbuf::~fdbuf()
{
	Detach();
}

fdbuf::int_type
fdbuf::underflow()
{
	int_type ret = traits_type::eof();

	if ( fd >= 0 && _M_mode & ios::in )
	{
		int_type rlen = Read( ibuf, buf_size );
		if ( rlen == 0 )
			return ret;

		ret = traits_type::to_int_type( *ibuf );
		setg( ibuf, ibuf, ibuf + rlen );
	}

	return ret;
}

fdbuf::int_type
fdbuf::overflow( int_type c )
{
	int_type ret_bad = traits_type::eof();
	
	if ( fd < 0 || ~_M_mode & ios::out || pbase() >= epptr() )
		return ret_bad;

	int_type ret_ok = traits_type::not_eof( c );
	int_type len = static_cast< char* >( pptr() )
		- static_cast< char* >( pbase() );

	if ( c != traits_type::eof() )
	{
		*pptr() = c;
		++len;
	}

	if ( !len )
		return ret_ok;

	if ( Write( pbase(), len ) == 0 )
		return ret_bad;

	setp( obuf, obuf + buf_size - 1 );

	return ret_ok;
}

void
fdbuf::Attach( int _fd, ios::openmode mode )
{
	if ( fd >= 0 )
		sync();

	fd = _fd;
	
	if ( mode & ios::in )
		setg( ibuf, ibuf, ibuf );
	if ( mode & ios::out )
		setp( obuf, obuf + buf_size - 1 );
}

void
fdbuf::Detach()
{
	Attach( -1, ios::openmode( 0 ) );
	
	setg( 0, 0, 0 );
	setp( 0, 0 );

	// not clean to set an implementation specific variable...
	_M_mode = ios::openmode( 0 );
}

fdbuf::int_type
fdbuf::Read( char_type* buf, int_type len )
{
	while ( true )
	{
		int rlen = read( fd, buf, len );

		if ( rlen < 0 )
		{
			if ( errno == EINTR )
				continue;
			Throw( system_error( "stream buffer underflow: "
								 "read() failed" ) );

			return 0;
		}

		return rlen;
	}
}

fdbuf::int_type
fdbuf::Write( char_type* buf, int_type len )
{
	while ( true )
	{
		int rlen = write( fd, buf, len );
			
		if ( rlen < 0 )
		{
			if ( errno == EINTR )
				continue;
			Throw( system_error( "stream buffer overflow: "
								 "write() failed" ) );

			return 0;
		}
		
		return rlen;
	}
}

}
