/*
 * The UDS Collection
 * Copyright (C) 2000, 2001, 2002, 2003  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <sys/file.h>
#include <uds/log.hh>
#include <uds/stdexcpt.hh>
#include <ctime>
#include <unistd.h>


namespace uds
{

// Logger implementation

Logger::Logger( loglev_t s, loglev_t p ) : ll_sys( s ), ll_priv( p ),
	flags( 0 )
{
}

Logger::~Logger()
{
	CloseLog();
}

void
Logger::OpenLog( const string& id, const char* file, flag_t f )
{
	ident = id;
	flags = f;
	
	if ( log.IsOpen() )
		log.Close();
	log.Open( file, ios::out | ios::app, FileStream::create );
	logfile = file;
}

void
Logger::OpenLog( const string& id, int opt, int fac )
{
	ident = id;
	openlog( ident.c_str(), opt, fac );
}

void
Logger::OpenLog( const string& id, const char* file, int opt, int fac, flag_t f )
{
	OpenLog( id, file, f );
	OpenLog( id, opt, fac );
}

void
Logger::CloseLog()
{
	if ( log.IsOpen() )
		log.Close();
	closelog();
}

void
Logger::operator () ( loglev_t p, const string& m ) const
{
	if ( p < ll_sys )
		syslog( p <? LOG_DEBUG, m.c_str() );

	if ( p < ll_priv && log.IsOpen() )
	{
		// lock logfile
		CkStdCall( flock( log.File(), LOCK_EX ) );
		
		// print stuff
		
		if ( flags & print_time )
		{
			time_t t = time( 0 );
			log<<string( ctime( &t ) ).substr( 4, 16 );
		}
		if ( flags & use_ident )
			log<<ident;
		if ( flags & log_pid )
			log<<'['<<getpid()<<']';
		if ( flags & print_pri )
			log<<'<'<<static_cast< int >( p )<<'>';
		if ( flags & ( use_ident | log_pid ) )
			log<<": ";
		
		if ( flags & no_newlines )
			for ( size_t i = 0; i < m.size(); ++i )
			{
				char c = m[i];
				log<<( ( c == '\n' ) ? ' ' : c );
			}
		else if ( flags & add_tabs )
		{
			size_t i = 0;
			for ( ; i < m.size() - 1; ++i )
			{
				char c = m[i];
				log<<c;
				if ( c == '\n' )
					log<<'\t';
			}
			log<<m[i];
		}
		else
			log<<m;
		
		if ( m[m.size() - 1] != '\n' )
			log<<endl;
		
		log.flush();
		
		// unlock logfile
		CkStdCall( flock( log.File(), LOCK_UN ) );
	}
}

}
