/*
 * The UDS Collection
 * Copyright (C) 2000, 2001, 2002, 2003  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <netinet/in.h>
#include <sys/socket.h>
#include <uds/packetsocket.hh>
#include <uds/stdexcpt.hh>
#include <netdb.h>
#include <unistd.h>


namespace uds
{

namespace
{

const int max_msglen = 2 << 12;

void
bind_udp_socket( int sock, unsigned short p )
{
	sockaddr_in si;
	init_inet_addr( si, p );
	CkStdCall( bind( sock, reinterpret_cast< sockaddr* >( &si ),
					 sizeof( si ) ));
}

}


PacketSocket::PacketSocket()
{
}

PacketSocket::PacketSocket( int fd )
	: Socket_base( fd )
{
}
	
PacketSocket::~PacketSocket()
{
	priv_close();
}
	
void
PacketSocket::Close()
{
	priv_close();
	Socket_base::Close();
}

void
PacketSocket::priv_close()
{
	if ( Valid() )
		close( Sock() );

}


// UDPSocket implementation

UDPSocket::UDPSocket( bool c )
{
	if ( c )
		Create();
}

UDPSocket::UDPSocket( int fd )
	: PacketSocket( fd )
{
}

UDPSocket::~UDPSocket()
{
}

void
UDPSocket::Create()
{
	Attach( CrUDPSocket() );
}

void
UDPSocket::Bind( unsigned short p )
{
	if ( Sock() < 0 )
		Create();
	
	bind_udp_socket( Sock(), p );
}

void
UDPSocket::SetDTarget( const char* h, unsigned short p )
{
	sockaddr_in si;
	init_inet_addr( si, h, p );
	CkStdCall( connect( Sock(), reinterpret_cast< sockaddr* >( &si ),
						sizeof( si ) ));
}

void
UDPSocket::UnsetDTarget()
{
	CkStdCall( connect( Sock(), 0, 0 ) );
}

void
UDPSocket::Send( const string& d, int f )
{
	CkStdCall( send( Sock(), d.c_str(), d.size(), f ) );
}

void
UDPSocket::Send( const string& d, const char* h, unsigned short p, int f )
{
	sockaddr_in si;
	init_inet_addr( si, h, p );
	CkStdCall( sendto( Sock(), d.c_str(), d.size(), f,
					   reinterpret_cast< sockaddr* >( &si ), sizeof( si ) ));
}

void
UDPSocket::Recv( string& d, int f )
{
	char msg[max_msglen];

	int sz = CkStdCall( recv( Sock(), msg, max_msglen, f ) );
	d.assign( msg, sz );
}

void
UDPSocket::Recv( string& d, string& h, int f )
{
	char msg[max_msglen];
	sockaddr_in si;
	socklen_t si_len;

	int sz = CkStdCall( recvfrom( Sock(), msg, max_msglen, f,
								  reinterpret_cast< sockaddr* >( &si ),
								  &si_len ) );
	d.assign( msg, sz );

	hostent* he = gethostbyaddr( &si.sin_addr, sizeof( si.sin_addr ), AF_INET );
	
	if ( he )
	{
		h = he->h_name;
		return;
	}
	
}


// misc. implementation

int
CrUDPSocket()
{
	return CkStdCall( socket( PF_INET, SOCK_DGRAM, 0 ) );
}

}
