/*
 * The UDS Collection
 * Copyright (C) 2000, 2001, 2002, 2003  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/random.hh>
#include <uds/stdexcpt.hh>
#include <uds/time.hh>
#include <fcntl.h>
#include <cmath>
#include <unistd.h>


namespace uds
{

namespace
{

unsigned
file_seed( const char* fn )
{
	unsigned randx;
	int fd = CkStdCall( open( fn, 0 ) );
	CkStdCall( read( fd, &randx, sizeof( randx ) ));
	close( fd );
	return randx;
}

}


// Random implementation

Random::Random( unsigned s ) : randx( s )
{
}

Random::~Random()
{
}

void
Random::Seed( unsigned s, bool a )
{
	randx = ( a ? randx : 0 ) + s;
}

unsigned
Random::draw()
{
	return randx = ( randx * 1103515245 + 12345 ) % Max();
}

double
Random::fdraw()
{
	return static_cast< double >( draw() ) / Max();
}


// misc. implementation

unsigned
TimeSeed()
{
	TimeVal tv = Time();
	return tv.sec ^ tv.usec ^ getpid();
}

unsigned
RandomSeed()
{
	return file_seed( "/dev/random" );
}

unsigned
URandomSeed()
{
	return file_seed( "/dev/urandom" );
}


// URandom implementation

URandom::URandom( unsigned m, unsigned s ) : Random( s ), max( m )
{
}

URandom::~URandom()
{
}

unsigned
URandom::Draw()
{
	unsigned r = static_cast< unsigned >( fdraw() * max );
	return ( r == max ) ? r - 1 : r;
}

}
