/*
 * The UDS Collection
 * Copyright (C) 2000, 2001, 2002, 2003  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include "config.h"
#ifdef HAVE_SOCKET

#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <uds/stdexcpt.hh>
#include <uds/sockbase.hh>
#include <fcntl.h>
#include <netdb.h>


namespace uds
{

// Socket implementation

Socket_base::Socket_base()
	: sock( -1 )
{
}

Socket_base::Socket_base( int fd )
	: sock( fd )
{
}

Socket_base::~Socket_base()
{
	Close();
}

void
Socket_base::Listen( unsigned n )
{
	CkStdCall( listen( sock, n ) );
}

int
Socket_base::Accept()
{
	return CkStdCall( accept( sock, 0, 0 ) );
}

void
Socket_base::Close()
{
	sock = -1;
}

void
Socket_base::Attach( int s )
{
	Close();
	sock = s;
}

bool
Socket_base::Block() const
{
	return CkStdCall( fcntl( sock, F_GETFL ) ) & ~O_NONBLOCK;
}

void
Socket_base::Block( bool f )
{
	CkStdCall( fcntl( sock, F_SETFL, f ? 0 : O_NONBLOCK ) );
}


// misc. implementation

void
init_local_addr( sockaddr_un& name, const char* filename )
{
	name.sun_family = AF_LOCAL;
	strncpy( name.sun_path, filename, sizeof( name.sun_path ) );
}

void
init_inet_addr( sockaddr_in& name, unsigned short p )
{
	name.sin_family = AF_INET;
	name.sin_port = htons( p );
	name.sin_addr.s_addr = htonl( INADDR_ANY );
}

void
init_inet_addr( sockaddr_in& name, const char* n, unsigned short p )
{
	hostent* hostinfo;
	
	name.sin_family = AF_INET;
	name.sin_port = htons( p );
	hostinfo = gethostbyname( n );
	if ( !hostinfo )
		Throw( system_error( "unable to resolve host name" ) );
	name.sin_addr = *reinterpret_cast< in_addr* >( hostinfo->h_addr );
}

}

#endif // HAVE_SOCKET
