/*
 * The UDS Collection
 * Copyright (C) 2000, 2001, 2002, 2003  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <time.hh>


namespace uds
{

// TimeVal implementation

TimeVal::TimeVal( double t )
{
	sec = static_cast< int >( t );
	usec = static_cast< int >( ( t - sec ) * 1e6 );
}

long long
TimeVal::to_int64() const
{
	return static_cast< long long >( sec ) * 1000000 + usec;
}

TimeVal&
TimeVal::operator += ( const TimeVal& rhs )
{
	sec += rhs.sec;
	usec += rhs.usec;
	return *this;
}

TimeVal&
TimeVal::operator -= ( const TimeVal& rhs )
{
	sec -= rhs.sec;
	usec -= rhs.usec;
	return *this;
}


// TimeSpec implementation

TimeSpec::TimeSpec( double t )
{
	sec = static_cast< int >( t );
	nsec = static_cast< int >( ( t - sec ) * 1e9 );
}

void
TimeSpec::Now()
{
	gettimeofday( reinterpret_cast< timeval* >( this ), 0 );
	nsec *= 1000;
}

TimeSpec&
TimeSpec::operator += ( const TimeSpec& rhs )
{
	sec += rhs.sec;
	nsec += rhs.nsec;
	return *this;
}

TimeSpec&
TimeSpec::operator -= ( const TimeSpec& rhs )
{
	sec -= rhs.sec;
	nsec -= rhs.nsec;
	return *this;
}

}
